/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#include "qubopt.h"

#include "QUB_NodeArray.h"
using namespace std;

void QUB_NodeArray::addNode(int i)
{
	if ( i == 1 )
		nodes[i] = root.appendChild("");
	else if ( i % 2 )
		nodes[i] = nodes[i/2].parent().insertChild(nodes[i/2], "");
	else
		nodes[i] = nodes[i/2].appendChild("");
}

void QUB_NodeArray::removeNode(int i)
{
	QTR_Impl* node = nodes[i].getImpl();
	QTR_Impl* parent = nodes[i].parent().getImpl();
	QTR_Impl* presib = NULL;

	nodes[i] = QUB_Tree();

	if ( i>1 && i%2 )
		presib = nodes[i/2].getImpl();

	QTR_RemoveChild( parent, presib, node );
}
		

void QUB_NodeArray::resize(int n)
{
	int cap, i;

	for ( cap=1; cap<=n; cap<<=1 )
		;
	
	for ( i=siz; i>n; --i )
		removeNode(i);
	nodes.resize( cap );
	for ( i=siz+1; i<=n; ++i )
		addNode(i);
	siz = n;
}



void      __cdecl QUB_NodeStash::Free()
{
	delete this;
}

QTR_Impl* __cdecl QUB_NodeStash::_getRoot()
{
	QTR_Impl *impl = nodes.getRoot().getImpl();
	QTR_INCREF( impl );
	return impl;
}

int       __cdecl QUB_NodeStash::request()
{
	int key;
	if ( keys.size() ) {
		set<int>::iterator ki = keys.begin();
		key = *ki;
		keys.erase(ki);
	}
	else {
		key = nodes.size();
		nodes.resize(key+1);
	}
	return key;
}

void      __cdecl QUB_NodeStash::replace(int key)
{
	keys.insert(key);

	set<int>::reverse_iterator ki = keys.rbegin();
	int nkey = keys.size();
	int maxnode = nodes.size() - 1;

	while ( nkey && (*ki == maxnode) ) {
		keys.erase(*ki);
		--nkey;
		nodes.resize(maxnode);
		--maxnode;
		ki = keys.rbegin();
	}
}

QTR_Impl* __cdecl QUB_NodeStash::_get(int key)
{
	QTR_Impl *impl = nodes[key].getImpl();
	QTR_INCREF( impl );
	return impl;
}

QUB_Tree QUB_NodeStash::getRoot()
{
	return nodes.getRoot();
}

QUB_Tree QUB_NodeStash::get(int key)
{
	return nodes[key];
}

